@@ -28,7 +28,7 @@ from functools import wraps
 import dbus
 import gettext
 import gobject
-import gtk
+import gtk  # pylint: disable=W0403
 import webkit
 import xdg
 
@@ -125,7 +125,7 @@ class LabeledEntry(gtk.Entry):
         self.is_password = is_password
         self.warning = None
 
-        super(LabeledEntry, self).__init__(*args, **kwargs)
+        gtk.Entry.__init__(self, *args, **kwargs)
 
         self.set_width_chars(DEFAULT_WIDTH)
         self._set_label(self, None)
@@ -161,7 +161,7 @@ class LabeledEntry(gtk.Entry):
 
     def get_text(self):
         """Get text only if it's not the label nor empty."""
-        result = super(LabeledEntry, self).get_text()
+        result = gtk.Entry.get_text(self)
         if result == self.label or result.isspace():
             result = ''
         return result
@@ -252,6 +252,8 @@ class UbuntuSSOClientGUI(object):
         self.tc_uri = tc_uri
         self.help_text = help_text
         self.close_callback = close_callback
+        self.user_email = None
+        self.user_password = None
 
         ui_filename = get_data_file('ui.glade')
         builder = gtk.Builder()
@@ -319,13 +321,13 @@ class UbuntuSSOClientGUI(object):
         self._append_page(self._build_login_page())
         self._append_page(self._build_request_password_token_page())
         self._append_page(self._build_set_new_password_page())
+        self._append_page(self._build_verify_email_page())
 
         window_size = None
         if not login_only:
             window_size = (550, 500)
             self._append_page(self._build_enter_details_page())
             self._append_page(self._build_tc_page())
-            self._append_page(self._build_verify_email_page())
             self.login_button.grab_focus()
             self._set_current_page(self.enter_details_vbox)
         else:
@@ -357,6 +359,8 @@ class UbuntuSSOClientGUI(object):
              self._filter_by_app_name(self.on_logged_in),
             'LoginError':
              self._filter_by_app_name(self.on_login_error),
+            'UserNotValidated':
+             self._filter_by_app_name(self.on_user_not_validated),
             'PasswordResetTokenSent':
              self._filter_by_app_name(self.on_password_reset_token_sent),
             'PasswordResetError':
@@ -417,8 +421,8 @@ class UbuntuSSOClientGUI(object):
 
             match = self.bus.add_signal_receiver(method, signal_name=signal,
                                                  dbus_interface=iface)
-            logger.info('Connecting signal %r with method %r at iface %r.' \
-                        'Match: %r', signal, method, iface, match)
+            logger.debug('Connecting signal %r with method %r at iface %r.' \
+                         'Match: %r', signal, method, iface, match)
             self._signals_receivers[(iface, signal)] = method
 
     def _debug(self, *args, **kwargs):
@@ -725,8 +729,8 @@ class UbuntuSSOClientGUI(object):
         remove = self.bus.remove_signal_receiver
         for (iface, signal) in self._signals_receivers.keys():
             method = self._signals_receivers.pop((iface, signal))
-            logger.info('Removing signal %r with method %r at iface %r.',
-                        signal, method, iface)
+            logger.debug('Removing signal %r with method %r at iface %r.',
+                         signal, method, iface)
             remove(method, signal_name=signal, dbus_interface=iface)
 
         # hide the main window
@@ -804,6 +808,8 @@ class UbuntuSSOClientGUI(object):
             return
 
         self._set_current_page(self.processing_vbox)
+        self.user_email = email1
+        self.user_password = password1
 
         logger.info('Calling register_user with email %r, password <hidden>,' \
                     ' captcha_id %r and captcha_solution %r.', email1,
@@ -832,8 +838,8 @@ class UbuntuSSOClientGUI(object):
             self.email_token_entry.set_warning(self.FIELD_REQUIRED)
             return
 
-        email = self.email1_entry.get_text()
-        password = self.password1_entry.get_text()
+        email = self.user_email
+        password = self.user_password
         f = self.backend.validate_email
         logger.info('Calling validate_email with email %r, password <hidden>' \
                     ', app_name %r and email_token %r.', email, self.app_name,
@@ -871,6 +877,8 @@ class UbuntuSSOClientGUI(object):
           reply_handler=NO_OP, error_handler=NO_OP)
 
         self._set_current_page(self.processing_vbox)
+        self.user_email = email
+        self.user_password = password
 
     def on_login_back_button_clicked(self, *args, **kwargs):
         """User wants to go to the previous page."""
@@ -1060,6 +1068,11 @@ class UbuntuSSOClientGUI(object):
         self._gtk_signal_log.append((SIG_LOGIN_FAILED, self.app_name,
                                      msg))
 
+    @log_call
+    def on_user_not_validated(self, app_name, email, *args, **kwargs):
+        """User was not validated."""
+        self.on_user_registered(app_name, email)
+
     @log_call
     def on_password_reset_token_sent(self, app_name, email, *args, **kwargs):
         """Password reset token was successfully sent."""
