@@ -152,6 +152,9 @@ class FakedAuthentications(object):
 class FakedAccounts(object):
     """Fake the accounts service."""
 
+    def __init__(self):
+        self.preferred_email = EMAIL
+
     def validate_email(self, email_token):
         """Fake the email validation. Return a fix result."""
         if email_token is None:
@@ -164,6 +167,17 @@ class FakedAccounts(object):
         else:
             return STATUS_EMAIL_OK
 
+    # pylint: disable=E0202, C0103
+
+    def me(self):
+        """Fake the 'me' information."""
+        return {u'username': u'Wh46bKY',
+                u'preferred_email': self.preferred_email,
+                u'displayname': u'',
+                u'unverified_emails': [u'aaaaaa@example.com'],
+                u'verified_emails': [],
+                u'openid_identifier': u'Wh46bKY'}
+
 
 class FakedSSOServer(object):
     """Fake an SSO server."""
@@ -279,6 +293,29 @@ class SSOLoginProcessorTestCase(TestCase, MockerTestCase):
         result = self.processor.login(**self.login_kwargs)
         self.assertEqual(TOKEN, result, 'authentication was successful.')
 
+    # is_validated
+
+    def test_is_validated(self):
+        """If preferred email is not None, user is validated."""
+        result = self.processor.is_validated(token=TOKEN)
+        self.assertTrue(result, 'user must be validated.')
+
+    def test_is_not_validated(self):
+        """If preferred email is None, user is not validated."""
+        service = FakedSSOServer(None, None)
+        service.accounts.preferred_email = None
+        result = self.processor.is_validated(sso_service=service,
+                                             token=TOKEN)
+        self.assertFalse(result, 'user must not be validated.')
+
+    def test_is_not_validated_empty_result(self):
+        """If preferred email is None, user is not validated."""
+        service = FakedSSOServer(None, None)
+        service.accounts.me = lambda: {}
+        result = self.processor.is_validated(sso_service=service,
+                                             token=TOKEN)
+        self.assertFalse(result, 'user must not be validated.')
+
     # validate_email
 
     def test_validate_email_if_status_ok(self):
@@ -380,12 +417,13 @@ class SsoDbusTestCase(TestCase):
         mockbus._register_object_path(ARGS)
         self.mockprocessorclass = None
 
-        def ksc(k, val):
+        def ksc(k, val, callback, *cb_args):
             """Assert over token and app_name."""
             self.assertEqual(k, APP_NAME)
             self.assertEqual(val, TOKEN)
             self.keyring_was_set = True
             self.keyring_values = k, val
+            callback(*cb_args)
 
         self.patch(ubuntu_sso.main, "keyring_store_credentials", ksc)
         self.keyring_was_set = False
@@ -519,8 +557,11 @@ class SsoDbusTestCase(TestCase):
     def test_login(self):
         """Test that the login method works ok."""
         d = Deferred()
-        self.create_mock_processor().login(EMAIL, PASSWORD, TOKEN_NAME)
+        processor = self.create_mock_processor()
+        processor.login(EMAIL, PASSWORD, TOKEN_NAME)
         self.mocker.result(TOKEN)
+        processor.is_validated(TOKEN)
+        self.mocker.result(True)
         self.patch(ubuntu_sso.main, "blocking", self.fake_ok_blocking)
         self.mocker.replay()
 
@@ -535,13 +576,40 @@ class SsoDbusTestCase(TestCase):
                           sso_login_processor_class=self.mockprocessorclass)
         self.patch(client, "LoggedIn", verify)
         self.patch(client, "LoginError", d.errback)
+        self.patch(client, "UserNotValidated", d.errback)
+        client.login(APP_NAME, EMAIL, PASSWORD)
+        return d
+
+    def test_login_user_not_validated(self):
+        """Test that the login sends EmailNotValidated signal."""
+        d = Deferred()
+        processor = self.create_mock_processor()
+        processor.login(EMAIL, PASSWORD, TOKEN_NAME)
+        self.mocker.result(TOKEN)
+        processor.is_validated(TOKEN)
+        self.mocker.result(False)
+        self.patch(ubuntu_sso.main, "blocking", self.fake_ok_blocking)
+        self.mocker.replay()
+
+        def verify(app_name, email):
+            """The actual test."""
+            self.assertEqual(app_name, APP_NAME)
+            self.assertEqual(email, EMAIL)
+            self.assertFalse(self.keyring_was_set, "Keyring should not be set")
+            d.callback("Ok")
+
+        client = SSOLogin(self.mockbusname,
+                          sso_login_processor_class=self.mockprocessorclass)
+        self.patch(client, "LoggedIn", d.errback)
+        self.patch(client, "LoginError", d.errback)
+        self.patch(client, "UserNotValidated", verify)
         client.login(APP_NAME, EMAIL, PASSWORD)
         return d
 
     def test_login_error(self):
         """Test that the login method fails as expected."""
         d = Deferred()
-        self.mockprocessorclass = self.mocker.mock()
+        self.create_mock_processor()
         self.patch(ubuntu_sso.main, "blocking", self.fake_err_blocking)
 
         def fake_gtn(*args):
@@ -562,6 +630,7 @@ class SsoDbusTestCase(TestCase):
                           sso_login_processor_class=self.mockprocessorclass)
         self.patch(client, "LoggedIn", d.errback)
         self.patch(client, "LoginError", verify)
+        self.patch(client, "UserNotValidated", d.errback)
         client.login(APP_NAME, EMAIL, PASSWORD)
         return d
 
@@ -591,7 +660,7 @@ class SsoDbusTestCase(TestCase):
     def test_validate_email_error(self):
         """Test that the validate_email method fails as expected."""
         d = Deferred()
-        self.mockprocessorclass = self.mocker.mock()
+        self.create_mock_processor()
         self.patch(ubuntu_sso.main, "blocking", self.fake_err_blocking)
 
         def fake_gtn(*args):
@@ -802,7 +871,7 @@ class ExceptToErrdictTestCase(TestCase):
         self.assertEqual(result["errtype"], e.__class__.__name__)
 
 
-class KeyringCredentialsTestCase(MockerTestCase):
+class KeyringCredentialsTestCase(TestCase, MockerTestCase):
     """Check the functions that access the keyring."""
 
     # Invalid name (should match ([a-z_][a-z0-9_]*|[A-Z_][A-Z0-9_]*)$)
@@ -810,15 +879,19 @@ class KeyringCredentialsTestCase(MockerTestCase):
 
     def test_keyring_store_cred(self):
         """Verify the method that stores credentials."""
+        idle_add = lambda f, *args, **kwargs: f(*args, **kwargs)
+        self.patch(gobject, "idle_add", idle_add)
         token_value = TOKEN
         mockKeyringClass = self.mocker.replace("ubuntu_sso.keyring.Keyring")
         mockKeyringClass(APP_NAME)
         mockKeyring = self.mocker.mock()
+        callback = self.mocker.mock()
         self.mocker.result(mockKeyring)
         mockKeyring.set_ubuntusso_attr(token_value)
+        callback(1, 2, 3)
         self.mocker.replay()
 
-        keyring_store_credentials(APP_NAME, token_value)
+        keyring_store_credentials(APP_NAME, token_value, callback, 1, 2, 3)
 
     def test_keyring_get_cred(self):
         """The method returns the right token."""
@@ -1367,6 +1440,24 @@ class PingServerTestCase(TestCase):
         self.assertEqual(self.calls[0][0], 'CredentialsError')
         self.assertEqual(self.calls[0][1][0], APP_NAME)
 
+    def test_credentials_are_not_stored_if_ping_failed(self):
+        """Credentials are not stored if the ping fails."""
+
+        def fail(*args, **kwargs):
+            """Raise an exception."""
+            self.args = AssertionError((args, kwargs))
+            # pylint: disable=E0702
+            raise self.args
+
+        self.patch(self.client, '_ping_url', fail)
+        self._patch('clear_token')
+
+        self.client._login_success_cb(None, APP_NAME, EMAIL)
+
+        self.assertEqual(len(self.calls), 1)
+        self.assertEqual(self.calls[0][0], 'clear_token')
+        self.assertEqual(self.calls[0][1][0], APP_NAME)
+
 
 class EnvironOverridesTestCase(TestCase):
     """Some URLs can be set from the environment for testing/QA purposes."""
